@@ -23,6 +23,27 @@
     Change History (most recent first):
     
 $Log: ThirdPage.cpp,v $
+Revision 1.27  2005/10/05 21:41:45  herscher
+<rdar://problem/4190104> Use "application/octet-stream" to determine if CUPS shared queue supports raw
+
+Revision 1.26  2005/07/11 20:17:15  shersche
+<rdar://problem/4124524> UI fixes associated with CUPS printer workaround fix.
+
+Revision 1.25  2005/07/07 17:53:20  shersche
+Fix problems associated with the CUPS printer workaround fix.
+
+Revision 1.24  2005/06/30 18:02:54  shersche
+<rdar://problem/4124524> Workaround for Mac OS X Printer Sharing bug
+
+Revision 1.23  2005/04/18 02:33:47  shersche
+<rdar://problem/4091216> Default printer option cannot be deselected
+
+Revision 1.22  2005/04/13 17:46:22  shersche
+<rdar://problem/4082122> Generic PCL not selected when printers advertise multiple text records
+
+Revision 1.21  2005/03/30 02:09:55  shersche
+Auto-resize the column width to account for differing fonts and font sizes
+
 Revision 1.20  2005/03/05 02:27:45  shersche
 <rdar://problem/4030388> Generic drivers don't do color
 
@@ -183,7 +204,7 @@ CThirdPage::CThirdPage()
 	require_noerr(err, exit);
 
 	//
-	// and lastly load our own special generic printer defs
+	// load our own special generic printer defs
 	//
 	err = LoadGenericPrintDriverDefs( m_manufacturers );
 	require_noerr( err, exit );
@@ -231,7 +252,7 @@ CThirdPage::~CThirdPage()
 //
 // ----------------------------------------------------
 void
-CThirdPage::SelectMatch(Printer * printer, Service * service, Manufacturers & manufacturers, Manufacturer * manufacturer, Model * model)
+CThirdPage::SelectMatch(Printer * printer, Service * service, Manufacturer * manufacturer, Model * model)
 {
 	LVFINDINFO	info;
 	int			nIndex;
@@ -240,8 +261,6 @@ CThirdPage::SelectMatch(Printer * printer, Service * service, Manufacturers & ma
 	check( manufacturer != NULL );
 	check( model != NULL );
 
-	PopulateUI( manufacturers );
-
 	//
 	// select the manufacturer
 	//
@@ -276,6 +295,15 @@ CThirdPage::SelectMatch(Printer * printer, Service * service, Manufacturers & ma
 }
 
 
+void
+CThirdPage::SelectMatch(Manufacturers & manufacturers, Printer * printer, Service * service, Manufacturer * manufacturer, Model * model)
+{
+	PopulateUI( manufacturers );
+
+	SelectMatch( printer, service, manufacturer, model );
+}
+
+
 // --------------------------------------------------------
 // CopyPrinterSettings
 //
@@ -824,7 +852,7 @@ CThirdPage::LoadGenericPrintDriverDefs( Manufacturers & manufacturers )
 
 	// First try and find our generic driver names
 
-	iter = manufacturers.find(L"HP");
+	iter = m_manufacturers.find(L"HP");
 	require_action( iter != manufacturers.end(), exit, err = kUnknownErr );
 	manufacturer = iter->second;
 
@@ -1028,7 +1056,7 @@ CThirdPage::NormalizeManufacturerName( const CString & name )
 // MatchManufacturer and MatchModel in turn.
 //
 
-OSStatus CThirdPage::MatchPrinter(Manufacturers & manufacturers, Printer * printer, Service * service)
+OSStatus CThirdPage::MatchPrinter(Manufacturers & manufacturers, Printer * printer, Service * service, bool useCUPSWorkaround)
 {
 	CString					normalizedProductName;
 	Manufacturer		*	manufacturer		=	NULL;
@@ -1039,20 +1067,27 @@ OSStatus CThirdPage::MatchPrinter(Manufacturers & manufacturers, Printer * print
 	CString					text;
 	OSStatus				err					=	kNoErr;
 
+	check( printer );
+	check( service );
+
+	Queue * q = service->SelectedQueue();
+
+	check( q );
+
 	//
 	// first look to see if we have a usb_MFG descriptor
 	//
-	if (service->usb_MFG.GetLength() > 0)
+	if ( q->usb_MFG.GetLength() > 0)
 	{
-		manufacturer = MatchManufacturer( manufacturers, ConvertToManufacturerName ( service->usb_MFG ) );
+		manufacturer = MatchManufacturer( manufacturers, ConvertToManufacturerName ( q->usb_MFG ) );
 	}
 
 	if ( manufacturer == NULL )
 	{
-		service->product.Remove('(');
-		service->product.Remove(')');
+		q->product.Remove('(');
+		q->product.Remove(')');
 
-		manufacturer = MatchManufacturer( manufacturers, ConvertToManufacturerName ( service->product ) );
+		manufacturer = MatchManufacturer( manufacturers, ConvertToManufacturerName ( q->product ) );
 	}
 	
 	//
@@ -1060,25 +1095,48 @@ OSStatus CThirdPage::MatchPrinter(Manufacturers & manufacturers, Printer * print
 	//
 	if ( manufacturer != NULL )
 	{
-		if (service->usb_MDL.GetLength() > 0)
+		if ( q->usb_MDL.GetLength() > 0 )
 		{
-			model = MatchModel ( manufacturer, ConvertToModelName ( service->usb_MDL ) );
+			model = MatchModel ( manufacturer, ConvertToModelName ( q->usb_MDL ) );
 		}
 
-		if ( ( model == NULL ) && ( service->product.GetLength() > 0 ) )
+		if ( ( model == NULL ) && ( q->product.GetLength() > 0 ) )
 		{
-			service->product.Remove('(');
-			service->product.Remove(')');
+			q->product.Remove('(');
+			q->product.Remove(')');
 
-			model = MatchModel ( manufacturer, ConvertToModelName ( service->product ) );
+			model = MatchModel ( manufacturer, ConvertToModelName ( q->product ) );
 		}
 
 		if ( model != NULL )
 		{
-			Manufacturers manufacturers;
-			
-			manufacturers[manufacturer->name] = manufacturer;
-			SelectMatch(printer, service, manufacturers, manufacturer, model);
+			// <rdar://problem/4124524> Offer Generic printers if printer advertises Postscript or PCL.  Workaround
+			// bug in OS X CUPS printer sharing by selecting Generic driver instead of matched printer.
+ 
+			bool hasGenericDriver = false;
+
+			if ( MatchGeneric( manufacturers, printer, service, &genericManufacturer, &genericModel ) )
+			{
+				hasGenericDriver = true;
+			}
+
+			// <rdar://problem/4190104> Use "application/octet-stream" to determine if CUPS 
+			// shared queue supports raw
+
+			if ( q->pdl.Find( L"application/octet-stream" ) != -1 )
+			{
+				useCUPSWorkaround = false;
+			}
+
+			if ( useCUPSWorkaround && printer->isSharedFromOSX && hasGenericDriver )
+			{
+				SelectMatch(manufacturers, printer, service, genericManufacturer, genericModel );
+			}
+			else
+			{
+				SelectMatch(manufacturers, printer, service, manufacturer, model);
+			}
+
 			found = true;
 		}
 	}
@@ -1091,26 +1149,18 @@ OSStatus CThirdPage::MatchPrinter(Manufacturers & manufacturers, Printer * print
 	{
 		text.LoadString(IDS_PRINTER_MATCH_GOOD);
 	}
-	else if ( MatchGeneric( printer, service, &genericManufacturer, &genericModel ) )
-	{
-		Manufacturers *	pManufacturers;
-		Manufacturers	manufacturers;
-		
-		text.LoadString(IDS_PRINTER_MATCH_MAYBE);
-		
-		if ( manufacturer )
+	else if ( MatchGeneric( manufacturers, printer, service, &genericManufacturer, &genericModel ) )
+	{	
+		if ( printer->isSharedFromOSX )
 		{
-			manufacturers[genericManufacturer->name]	= genericManufacturer;
-			manufacturers[manufacturer->name]			= manufacturer;
-
-			pManufacturers = &manufacturers;
+			text.LoadString(IDS_PRINTER_MATCH_GOOD);
 		}
 		else
 		{
-			pManufacturers = &m_manufacturers;
+			text.LoadString(IDS_PRINTER_MATCH_MAYBE);
 		}
 
-		SelectMatch( printer, service, *pManufacturers, genericManufacturer, genericModel );
+		SelectMatch( manufacturers, printer, service, genericManufacturer, genericModel );
 	}
 	else
 	{
@@ -1247,19 +1297,25 @@ CThirdPage::MatchModel(Manufacturer * manufacturer, const CString & name)
 // specifically
 //
 BOOL
-CThirdPage::MatchGeneric( Printer * printer, Service * service, Manufacturer ** manufacturer, Model ** model )
+CThirdPage::MatchGeneric( Manufacturers & manufacturers, Printer * printer, Service * service, Manufacturer ** manufacturer, Model ** model )
 {
 	CString	pdl;
 	BOOL	ok = FALSE;
 
 	DEBUG_UNUSED( printer );
 
-	Manufacturers::iterator iter = m_manufacturers.find( kGenericManufacturer );
-	require_action_quiet( iter != m_manufacturers.end(), exit, ok = FALSE );
+	check( service );
+
+	Queue * q = service->SelectedQueue();
+
+	check( q );
+
+	Manufacturers::iterator iter = manufacturers.find( kGenericManufacturer );
+	require_action_quiet( iter != manufacturers.end(), exit, ok = FALSE );
 
 	*manufacturer = iter->second;
 
-	pdl = service->pdl;
+	pdl = q->pdl;
 	pdl.MakeLower();
 
 	if ( pdl.Find( kPDLPCLKey ) != -1 )
@@ -1323,11 +1379,11 @@ OSStatus CThirdPage::OnInitPage()
 	// selection notice
 	//
 	header.LoadString(IDS_MANUFACTURER_HEADING);
-	m_manufacturerListCtrl.InsertColumn(0, header, LVCFMT_LEFT, 138);
+	m_manufacturerListCtrl.InsertColumn(0, header, LVCFMT_LEFT, -1 );
 	m_manufacturerSelected = NULL;
 
 	header.LoadString(IDS_MODEL_HEADING);
-	m_modelListCtrl.InsertColumn(0, header, LVCFMT_LEFT, 247);
+	m_modelListCtrl.InsertColumn(0, header, LVCFMT_LEFT, -1 );
 	m_modelSelected = NULL;
 
 	return (err);
@@ -1363,15 +1419,7 @@ CThirdPage::OnSetActive()
 	psheet = reinterpret_cast<CPrinterSetupWizardSheet*>(GetParent());
 	require_quiet( psheet, exit );
    
-	if ((m_manufacturerListCtrl.GetFirstSelectedItemPosition() != NULL) &&
-	    (m_modelListCtrl.GetFirstSelectedItemPosition() != NULL))
-	{
-		psheet->SetWizardButtons( PSWIZB_BACK|PSWIZB_NEXT );
-	}
-	else
-	{
-		psheet->SetWizardButtons( PSWIZB_BACK );
-	}
+	psheet->SetWizardButtons( PSWIZB_BACK );
 
 	printer = psheet->GetSelectedPrinter();
 	require_quiet( printer, exit );
@@ -1402,7 +1450,15 @@ CThirdPage::OnSetActive()
 	//
 	// and try and match the printer
 	//
-	MatchPrinter( m_manufacturers, printer, service );
+
+	if ( psheet->GetLastPage() == psheet->GetPage(1) )
+	{
+		MatchPrinter( m_manufacturers, printer, service, true );
+	}
+	else
+	{
+		SelectMatch(printer, service, m_manufacturerSelected, m_modelSelected);
+	}
 
 exit:
 
@@ -1410,6 +1466,22 @@ exit:
 }
 
 
+BOOL
+CThirdPage::OnKillActive()
+{
+	CPrinterSetupWizardSheet * psheet;
+
+	psheet = reinterpret_cast<CPrinterSetupWizardSheet*>(GetParent());
+	require_quiet( psheet, exit );   
+   
+	psheet->SetLastPage(this);
+
+exit:
+
+	return CPropertyPage::OnKillActive();
+}
+
+
 // -------------------------------------------------------
 // PopulateUI
 //
@@ -1431,6 +1503,8 @@ CThirdPage::PopulateUI(Manufacturers & manufacturers)
 		nIndex = m_manufacturerListCtrl.InsertItem(0, manufacturer->name);
 
 		m_manufacturerListCtrl.SetItemData(nIndex, (DWORD_PTR) manufacturer);
+
+		m_manufacturerListCtrl.SetColumnWidth( 0, LVSCW_AUTOSIZE_USEHEADER );
 	}
 
 	return 0;
@@ -1471,6 +1545,8 @@ void CThirdPage::OnLvnItemchangedManufacturer(NMHDR *pNMHDR, LRESULT *pResult)
 			int nItem = m_modelListCtrl.InsertItem( 0, model->displayName );
 
 			m_modelListCtrl.SetItemData(nItem, (DWORD_PTR) model);
+
+			m_modelListCtrl.SetColumnWidth( 0, LVSCW_AUTOSIZE_USEHEADER );
 		}
 
 		m_modelListCtrl.SetRedraw(TRUE);
@@ -1531,7 +1607,7 @@ void CThirdPage::OnBnClickedDefaultPrinter()
 	printer = psheet->GetSelectedPrinter();
 	require_quiet( printer, exit );
 
-	printer->deflt = m_defaultPrinterCtrl.GetState() ? true : false;
+	printer->deflt = ( m_defaultPrinterCtrl.GetCheck() == BST_CHECKED ) ? true : false;
 
 exit:
 
@@ -1543,6 +1619,7 @@ void CThirdPage::OnBnClickedHaveDisk()
 	CPrinterSetupWizardSheet	*	psheet;
 	Printer						*	printer;
 	Service						*	service;
+	Manufacturers					manufacturers;
 
 	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY|OFN_FILEMUSTEXIST, L"Setup Information (*.inf)|*.inf||", this);
 
@@ -1555,16 +1632,29 @@ void CThirdPage::OnBnClickedHaveDisk()
 	service = printer->services.front();
 	require_quiet( service, exit );
 
-	if ( dlg.DoModal() == IDOK )
+	for ( ;; )
 	{
-		Manufacturers	manufacturers;
-		CString			filename = dlg.GetPathName();
+		if ( dlg.DoModal() == IDOK )
+		{
+			CString filename = dlg.GetPathName();
 
-		LoadPrintDriverDefsFromFile( manufacturers, filename, true );
+			LoadPrintDriverDefsFromFile( manufacturers, filename, true );
    
-		PopulateUI( manufacturers );
+			// Sanity check
 
-		MatchPrinter( manufacturers, printer, service );
+			if ( manufacturers.size() > 0 )
+			{
+				PopulateUI( manufacturers );
+
+				MatchPrinter( manufacturers, printer, service, false );
+
+				break;
+			}
+		}
+		else
+		{
+			break;
+		}
 	}
 
 exit:
