@@ -1,10 +1,9 @@
-/*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+/* -*- Mode: C; tab-width: 4 -*-
+ *
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -25,6 +24,57 @@
     Change History (most recent first):
 
 $Log: mDNSUNP.c,v $
+Revision 1.32  2005/12/21 02:56:43  cheshire
+<rdar://problem/4243433> get_ifi_info() should fake ifi_index when SIOCGIFINDEX undefined
+
+Revision 1.31  2005/12/21 02:46:05  cheshire
+<rdar://problem/4243514> mDNSUNP.c needs to include <sys/param.h> on 4.4BSD Lite
+
+Revision 1.30  2005/11/29 20:03:02  mkrochma
+Wrapped sin_len with #ifndef NOT_HAVE_SA_LEN
+
+Revision 1.29  2005/11/12 02:23:10  cheshire
+<rdar://problem/4317680> mDNSUNP.c needs to deal with lame results from SIOCGIFNETMASK, SIOCGIFBRDADDR and SIOCGIFDSTADDR
+
+Revision 1.28  2005/10/31 22:09:45  cheshire
+Buffer "char addr6[33]" was seven bytes too small
+
+Revision 1.27  2005/06/29 15:54:21  cheshire
+<rdar://problem/4113742> mDNSResponder-107.1 does not work on FreeBSD
+Refine last checkin so that it (hopefully) doesn't break get_ifi_info() for every other OS
+
+Revision 1.26  2005/04/08 21:43:59  ksekar
+<rdar://problem/4083426>  mDNSPosix (v98) retrieve interface list bug on AMD64 architecture
+Submitted by Andrew de Quincey
+
+Revision 1.25  2005/04/08 21:37:57  ksekar
+<rdar://problem/3792767> get_ifi_info doesn't return IPv6 interfaces on Linux
+
+Revision 1.24  2005/04/08 21:30:16  ksekar
+<rdar://problem/4007457> Compiling problems with mDNSResponder-98 on Solaris/Sparc v9
+Patch submitted by Bernd Kuhls
+
+Revision 1.23  2004/12/01 04:25:05  cheshire
+<rdar://problem/3872803> Darwin patches for Solaris and Suse
+Provide daemon() for platforms that don't have it
+
+Revision 1.22  2004/11/30 22:37:01  cheshire
+Update copyright dates and add "Mode: C; tab-width: 4" headers
+
+Revision 1.21  2004/11/08 22:13:59  rpantos
+Create sockf6 lazily when v6 interface found.
+
+Revision 1.20  2004/10/16 00:17:01  cheshire
+<rdar://problem/3770558> Replace IP TTL 255 check with local subnet source address check
+
+Revision 1.19  2004/07/20 01:47:36  rpantos
+NOT_HAVE_SA_LEN applies to v6, too. And use more-portable s6_addr.
+
+Revision 1.18  2004/07/08 21:30:21  rpantos
+
+Revision 1.17  2004/06/25 00:26:27  rpantos
+Changes to fix the Posix build on Solaris.
+
 Revision 1.16  2004/03/20 05:37:09  cheshire
 Fix contributed by Terry Lambert & Alfred Perlstein:
 Don't use uint8_t -- it requires stdint.h, which doesn't exist on FreeBSD 4.x
@@ -91,6 +141,15 @@ First checkin
 #include <unistd.h>
 #include <stdio.h>
 
+/* Some weird platforms derived from 4.4BSD Lite (e.g. EFI) need the ALIGN(P)
+   macro, usually defined in <sys/param.h> or someplace like that, to make sure the
+   CMSG_NXTHDR macro is well-formed. On such platforms, the symbol NEED_ALIGN_MACRO
+   should be set to the name of the header to include to get the ALIGN(P) macro.
+*/
+#ifdef NEED_ALIGN_MACRO
+#include NEED_ALIGN_MACRO
+#endif
+
 /* Solaris defined SIOCGIFCONF etc in <sys/sockio.h> but 
    other platforms don't even have that include file.  So, 
    if we haven't yet got a definition, let's try to find 
@@ -109,12 +168,143 @@ First checkin
     #include <net/if_dl.h>
 #endif
 
+#if defined(AF_INET6) && HAVE_IPV6 && !HAVE_LINUX
+#include <netinet6/in6_var.h>
+#endif
+
+#if defined(AF_INET6) && HAVE_IPV6 && HAVE_LINUX
+#include <netdb.h>
+#include <arpa/inet.h>
+
+/* Converts a prefix length to IPv6 network mask */
+void plen_to_mask(int plen, char *addr) {
+	int i;
+	int colons=7; /* Number of colons in IPv6 address */
+	int bits_in_block=16; /* Bits per IPv6 block */
+	for(i=0;i<=colons;i++) {
+		int block, ones=0xffff, ones_in_block;
+		if(plen>bits_in_block) ones_in_block=bits_in_block;
+		else                   ones_in_block=plen;
+		block = ones & (ones << (bits_in_block-ones_in_block));
+		i==0 ? sprintf(addr, "%x", block) :
+ sprintf(addr, "%s:%x", addr, block);
+		plen -= ones_in_block;
+		}
+	}
+
+/* Gets IPv6 interface information from the /proc filesystem in linux*/
+struct ifi_info *get_ifi_info_linuxv6(int family, int doaliases)
+	{
+	struct ifi_info *ifi, *ifihead, **ifipnext;
+	FILE *fp;
+	char addr[8][5];
+	int flags, myflags, index, plen, scope;
+	char ifname[8], lastname[IFNAMSIZ];
+	char addr6[32+7+1]; /* don't forget the seven ':' */
+	struct addrinfo hints, *res0;
+	struct sockaddr_in6 *sin6;
+	struct in6_addr *addrptr;
+	int err;
+
+	res0=NULL;
+	ifihead = NULL;
+	ifipnext = &ifihead;
+	lastname[0] = 0;
+
+	if ((fp = fopen(PROC_IFINET6_PATH, "r")) != NULL) {
+		while (fscanf(fp,
+					  "%4s%4s%4s%4s%4s%4s%4s%4s %02x %02x %02x %02x %8s\n",
+					  addr[0],addr[1],addr[2],addr[3],
+					  addr[4],addr[5],addr[6],addr[7],
+					  &index, &plen, &scope, &flags, ifname) != EOF) {
+
+			myflags = 0;
+			if (strncmp(lastname, ifname, IFNAMSIZ) == 0) {
+				if (doaliases == 0)
+					continue;   /* already processed this interface */
+				myflags = IFI_ALIAS;
+				}
+			memcpy(lastname, ifname, IFNAMSIZ);
+			ifi = (struct ifi_info*)calloc(1, sizeof(struct ifi_info));
+			if (ifi == NULL) {
+				goto gotError;
+				}
+
+			*ifipnext = ifi;            /* prev points to this new one */
+			ifipnext = &ifi->ifi_next;  /* pointer to next one goes here */
+
+			sprintf(addr6, "%s:%s:%s:%s:%s:%s:%s:%s",
+					addr[0],addr[1],addr[2],addr[3],
+					addr[4],addr[5],addr[6],addr[7]);
+
+			/* Add address of the interface */
+			memset(&hints, 0, sizeof(hints));
+			hints.ai_family = AF_INET6;
+			hints.ai_flags = AI_NUMERICHOST;
+			err = getaddrinfo(addr6, NULL, &hints, &res0);
+			if (err) {
+				goto gotError;
+				}
+			ifi->ifi_addr = calloc(1, sizeof(struct sockaddr_in6));
+			if (ifi->ifi_addr == NULL) {
+				goto gotError;
+				}
+			memcpy(ifi->ifi_addr, res0->ai_addr, sizeof(struct sockaddr_in6));
+
+			/* Add netmask of the interface */
+			char ipv6addr[INET6_ADDRSTRLEN];
+			plen_to_mask(plen, ipv6addr);
+			ifi->ifi_netmask = calloc(1, sizeof(struct sockaddr_in6));
+			if (ifi->ifi_addr == NULL) {
+				goto gotError;
+				}
+			sin6=calloc(1, sizeof(struct sockaddr_in6));
+			addrptr=calloc(1, sizeof(struct in6_addr));
+			inet_pton(family, ipv6addr, addrptr);
+			sin6->sin6_family=family;
+			sin6->sin6_addr=*addrptr;
+			sin6->sin6_scope_id=scope;
+			memcpy(ifi->ifi_netmask, sin6, sizeof(struct sockaddr_in6));
+			free(sin6);
+
+
+			/* Add interface name */
+			memcpy(ifi->ifi_name, ifname, IFI_NAME);
+
+			/* Add interface index */
+			ifi->ifi_index = index;
+
+			/* If interface is in /proc then it is up*/
+			ifi->ifi_flags = IFF_UP;
+
+			freeaddrinfo(res0);
+			res0=NULL;
+			}
+		}
+	goto done;
+
+	gotError:
+	if (ifihead != NULL) {
+		free_ifi_info(ifihead);
+		ifihead = NULL;
+		}
+	if (res0 != NULL) {
+		freeaddrinfo(res0);
+		res0=NULL;
+		}
+	done:
+	return(ifihead);    /* pointer to first structure in linked list */
+	}
+#endif // defined(AF_INET6) && HAVE_IPV6 && HAVE_LINUX
 
 struct ifi_info *get_ifi_info(int family, int doaliases)
 {
     int                 junk;
     struct ifi_info     *ifi, *ifihead, **ifipnext;
-    int                 sockfd, len, lastlen, flags, myflags;
+    int                 sockfd, sockf6, len, lastlen, flags, myflags;
+#ifdef NOT_HAVE_IF_NAMETOINDEX
+    int                 index = 200;
+#endif
     char                *ptr, *buf, lastname[IFNAMSIZ], *cptr;
     struct ifconf       ifc;
     struct ifreq        *ifr, ifrcopy;
@@ -124,7 +314,12 @@ struct ifi_info *get_ifi_info(int family, int doaliases)
     struct sockaddr_in6 *sinptr6;
 #endif
 
-    sockfd = -1;
+#if defined(AF_INET6) && HAVE_IPV6 && HAVE_LINUX
+ if(family == AF_INET6) return get_ifi_info_linuxv6(family, doaliases);
+#endif
+
+	sockfd = -1;
+    sockf6 = -1;
     buf = NULL;
     ifihead = NULL;
     
@@ -163,10 +358,13 @@ struct ifi_info *get_ifi_info(int family, int doaliases)
     for (ptr = buf; ptr < buf + ifc.ifc_len; ) {
         ifr = (struct ifreq *) ptr;
 
-		len = GET_SA_LEN(ifr->ifr_addr);
-        ptr += sizeof(ifr->ifr_name) + len; /* for next one in buffer */
-    
-//        fprintf(stderr, "intf %d name=%s AF=%d\n", index, ifr->ifr_name, ifr->ifr_addr.sa_family);
+        /* Advance to next one in buffer */
+        if (sizeof(struct ifreq) > sizeof(ifr->ifr_name) + GET_SA_LEN(ifr->ifr_addr))
+            ptr += sizeof(struct ifreq);
+        else
+            ptr += sizeof(ifr->ifr_name) + GET_SA_LEN(ifr->ifr_addr);
+
+//      fprintf(stderr, "intf %p name=%s AF=%d\n", index, ifr->ifr_name, ifr->ifr_addr.sa_family);
         
         if (ifr->ifr_addr.sa_family != family)
             continue;   /* ignore if not desired address family */
@@ -199,7 +397,17 @@ struct ifi_info *get_ifi_info(int family, int doaliases)
 
         ifi->ifi_flags = flags;     /* IFF_xxx values */
         ifi->ifi_myflags = myflags; /* IFI_xxx values */
+#ifndef NOT_HAVE_IF_NAMETOINDEX
         ifi->ifi_index = if_nametoindex(ifr->ifr_name);
+#else
+        ifrcopy = *ifr;
+#ifdef SIOCGIFINDEX
+		if ( 0 >= ioctl(sockfd, SIOCGIFINDEX, &ifrcopy))
+            ifi->ifi_index = ifrcopy.ifr_index;
+        else
+#endif
+            ifi->ifi_index = index++;	/* SIOCGIFINDEX is broken on Solaris 2.5ish, so fake it */
+#endif
         memcpy(ifi->ifi_name, ifr->ifr_name, IFI_NAME);
         ifi->ifi_name[IFI_NAME-1] = '\0';
 /* end get_ifi_info2 */
@@ -214,12 +422,30 @@ struct ifi_info *get_ifi_info(int family, int doaliases)
                 }
                 memcpy(ifi->ifi_addr, sinptr, sizeof(struct sockaddr_in));
 
+#ifdef  SIOCGIFNETMASK
+				if (ioctl(sockfd, SIOCGIFNETMASK, &ifrcopy) < 0) goto gotError;
+				ifi->ifi_netmask = (struct sockaddr*)calloc(1, sizeof(struct sockaddr_in));
+				if (ifi->ifi_netmask == NULL) goto gotError;
+				sinptr = (struct sockaddr_in *) &ifrcopy.ifr_addr;
+				/* The BSD ioctls (including Mac OS X) stick some weird values in for sin_len and sin_family */
+#ifndef NOT_HAVE_SA_LEN
+				sinptr->sin_len    = sizeof(struct sockaddr_in);
+#endif
+				sinptr->sin_family = AF_INET;
+				memcpy(ifi->ifi_netmask, sinptr, sizeof(struct sockaddr_in));
+#endif
+
 #ifdef  SIOCGIFBRDADDR
                 if (flags & IFF_BROADCAST) {
                     if (ioctl(sockfd, SIOCGIFBRDADDR, &ifrcopy) < 0) {
                         goto gotError;
                     }
                     sinptr = (struct sockaddr_in *) &ifrcopy.ifr_broadaddr;
+					/* The BSD ioctls (including Mac OS X) stick some weird values in for sin_len and sin_family */
+#ifndef NOT_HAVE_SA_LEN
+					sinptr->sin_len    = sizeof( struct sockaddr_in );
+#endif
+					sinptr->sin_family = AF_INET;
                     ifi->ifi_brdaddr = (struct sockaddr*)calloc(1, sizeof(struct sockaddr_in));
                     if (ifi->ifi_brdaddr == NULL) {
                         goto gotError;
@@ -234,6 +460,11 @@ struct ifi_info *get_ifi_info(int family, int doaliases)
                         goto gotError;
                     }
                     sinptr = (struct sockaddr_in *) &ifrcopy.ifr_dstaddr;
+                    /* The BSD ioctls (including Mac OS X) stick some weird values in for sin_len and sin_family */
+#ifndef NOT_HAVE_SA_LEN
+					sinptr->sin_len    = sizeof( struct sockaddr_in );
+#endif
+					sinptr->sin_family = AF_INET;
                     ifi->ifi_dstaddr = (struct sockaddr*)calloc(1, sizeof(struct sockaddr_in));
                     if (ifi->ifi_dstaddr == NULL) {
                         goto gotError;
@@ -256,8 +487,24 @@ struct ifi_info *get_ifi_info(int family, int doaliases)
                 /* Some platforms (*BSD) inject the prefix in IPv6LL addresses */
                 /* We need to strip that out */
                 if (IN6_IS_ADDR_LINKLOCAL(&sinptr6->sin6_addr))
-                	sinptr6->sin6_addr.__u6_addr.__u6_addr16[1] = 0;
+                	sinptr6->sin6_addr.s6_addr[2] = sinptr6->sin6_addr.s6_addr[3] = 0;
                 memcpy(ifi->ifi_addr, sinptr6, sizeof(struct sockaddr_in6));
+
+#ifdef  SIOCGIFNETMASK_IN6
+				{
+				struct in6_ifreq ifr6;
+				if (sockf6 == -1)
+					sockf6 = socket(AF_INET6, SOCK_DGRAM, 0);
+				bzero(&ifr6, sizeof(ifr6));
+				memcpy(&ifr6.ifr_name,           &ifr->ifr_name, sizeof(ifr6.ifr_name          ));
+				memcpy(&ifr6.ifr_ifru.ifru_addr, &ifr->ifr_addr, sizeof(ifr6.ifr_ifru.ifru_addr));
+				if (ioctl(sockf6, SIOCGIFNETMASK_IN6, &ifr6) < 0) goto gotError;
+				ifi->ifi_netmask = (struct sockaddr*)calloc(1, sizeof(struct sockaddr_in6));
+				if (ifi->ifi_netmask == NULL) goto gotError;
+				sinptr6 = (struct sockaddr_in6 *) &ifr6.ifr_ifru.ifru_addr;
+				memcpy(ifi->ifi_netmask, sinptr6, sizeof(struct sockaddr_in6));
+				}
+#endif
             }
             break;
 #endif
@@ -282,6 +529,10 @@ done:
         junk = close(sockfd);
         assert(junk == 0);
     }
+    if (sockf6 != -1) {
+        junk = close(sockf6);
+        assert(junk == 0);
+    }
     return(ifihead);    /* pointer to first structure in linked list */
 }
 /* end get_ifi_info3 */
@@ -407,11 +658,11 @@ struct in_pktinfo
         if (cmptr->cmsg_level == IPPROTO_IP &&
             cmptr->cmsg_type == IP_RECVIF) {
             struct sockaddr_dl  *sdl = (struct sockaddr_dl *) CMSG_DATA(cmptr);
-            int nameLen = (sdl->sdl_nlen < IFI_NAME - 1) ? sdl->sdl_nlen : (IFI_NAME - 1);
-            pktp->ipi_ifindex = sdl->sdl_index;
 #ifndef HAVE_BROKEN_RECVIF_NAME
+            int nameLen = (sdl->sdl_nlen < IFI_NAME - 1) ? sdl->sdl_nlen : (IFI_NAME - 1);
             strncpy(pktp->ipi_ifname, sdl->sdl_data, nameLen);
 #endif
+            pktp->ipi_ifindex = sdl->sdl_index;
             assert(pktp->ipi_ifname[IFI_NAME - 1] == 0);
             // null terminated because of memset above
             continue;
@@ -438,7 +689,9 @@ struct in_pktinfo
 			struct in6_pktinfo *ip6_info = (struct in6_pktinfo*)CMSG_DATA(cmptr);
 			
             sin6->sin6_family   = AF_INET6;
+#ifndef NOT_HAVE_SA_LEN
             sin6->sin6_len      = sizeof(*sin6);
+#endif
             sin6->sin6_addr     = ip6_info->ipi6_addr;
             sin6->sin6_flowinfo = 0;
             sin6->sin6_scope_id = 0;
@@ -460,3 +713,53 @@ struct in_pktinfo
     return(n);
 #endif /* CMSG_FIRSTHDR */
 }
+
+// **********************************************************************************************
+
+// daemonize the process. Adapted from "Unix Network Programming" vol 1 by Stevens, section 12.4.
+// Returns 0 on success, -1 on failure.
+
+#ifdef NOT_HAVE_DAEMON
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/signal.h>
+
+int daemon(int nochdir, int noclose)
+    {
+	switch (fork())
+		{
+		case -1: return (-1);	// Fork failed
+		case 0:  break;			// Child -- continue
+		default: _exit(0);		// Parent -- exit
+		}
+
+	if (setsid() == -1) return(-1);
+
+	signal(SIGHUP, SIG_IGN);
+
+	switch (fork())				// Fork again, primarily for reasons of Unix trivia
+		{
+		case -1: return (-1);	// Fork failed
+		case 0:  break;			// Child -- continue
+		default: _exit(0);		// Parent -- exit
+		}
+
+	if (!nochdir) (void)chdir("/");
+	umask(0);
+
+	if (!noclose)
+		{
+		int fd = open("/dev/null", O_RDWR, 0);
+		if (fd != -1)
+			{
+			// Avoid unnecessarily duplicating a file descriptor to itself
+			if (fd != STDIN_FILENO) (void)dup2(fd, STDIN_FILENO);
+			if (fd != STDOUT_FILENO) (void)dup2(fd, STDOUT_FILENO);
+			if (fd != STDERR_FILENO) (void)dup2(fd, STDERR_FILENO);
+			if (fd != STDIN_FILENO && fd != STDOUT_FILENO && fd != STDERR_FILENO) 
+				(void)close (fd);
+			}
+		}
+	return (0);
+    }
+#endif /* NOT_HAVE_DAEMON */
