@@ -1,10 +1,9 @@
-/*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+/* -*- Mode: C; tab-width: 4 -*-
+ *
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -25,6 +24,22 @@
     Change History (most recent first):
 
 $Log: mDNSUNP.h,v $
+Revision 1.18  2005/04/08 21:37:57  ksekar
+<rdar://problem/3792767> get_ifi_info doesn't return IPv6 interfaces on Linux
+
+Revision 1.17  2004/12/17 19:32:43  cheshire
+Add missing semicolon
+
+Revision 1.16  2004/12/01 04:25:05  cheshire
+<rdar://problem/3872803> Darwin patches for Solaris and Suse
+Provide daemon() for platforms that don't have it
+
+Revision 1.15  2004/11/30 22:37:01  cheshire
+Update copyright dates and add "Mode: C; tab-width: 4" headers
+
+Revision 1.14  2004/10/16 00:17:01  cheshire
+<rdar://problem/3770558> Replace IP TTL 255 check with local subnet source address check
+
 Revision 1.13  2004/03/20 05:37:09  cheshire
 Fix contributed by Terry Lambert & Alfred Perlstein:
 Don't use uint8_t -- it requires stdint.h, which doesn't exist on FreeBSD 4.x
@@ -78,6 +93,10 @@ First checkin
 #include <net/if.h>
 #include <netinet/in.h>
 
+#ifdef HAVE_LINUX
+#include <linux/socket.h>
+#endif
+
 #ifdef  __cplusplus
     extern "C" {
 #endif
@@ -87,8 +106,12 @@ First checkin
 #endif
 
 #if !defined(_SS_MAXSIZE)
-    #define sockaddr_storage sockaddr
-#endif
+#if HAVE_IPV6
+#define sockaddr_storage sockaddr_in6
+#else
+#define sockaddr_storage sockaddr
+#endif // HAVE_IPV6	
+#endif // !defined(_SS_MAXSIZE)
 
 #ifndef NOT_HAVE_SA_LEN
 #define GET_SA_LEN(X) (sizeof(struct sockaddr) > ((struct sockaddr*)&(X))->sa_len ? \
@@ -128,11 +151,23 @@ struct ifi_info {
   short   ifi_myflags;          /* our own IFI_xxx flags */
   int     ifi_index;            /* interface index */
   struct sockaddr  *ifi_addr;   /* primary address */
+  struct sockaddr  *ifi_netmask;
   struct sockaddr  *ifi_brdaddr;/* broadcast address */
   struct sockaddr  *ifi_dstaddr;/* destination address */
   struct ifi_info  *ifi_next;   /* next of these structures */
 };
 
+#if defined(AF_INET6) && HAVE_IPV6 && HAVE_LINUX
+#define PROC_IFINET6_PATH "/proc/net/if_inet6"
+extern struct ifi_info  *get_ifi_info_linuxv6(int family, int doaliases);
+#endif
+	
+#if defined(AF_INET6) && HAVE_IPV6
+#define INET6_ADDRSTRLEN 46 /*Maximum length of IPv6 address */
+#endif
+	
+
+	
 #define IFI_ALIAS   1           /* ifi_addr is an alias */
 
 /* From the text (Stevens, section 16.6): */
@@ -145,6 +180,10 @@ extern struct ifi_info  *get_ifi_info(int family, int doaliases);
 /* returned by get_ifi_info and frees all the dynamic memory.' */
 extern void             free_ifi_info(struct ifi_info *);
 
+#ifdef NOT_HAVE_DAEMON
+extern int daemon(int nochdir, int noclose);
+#endif
+
 #ifdef  __cplusplus
     }
 #endif
